package MostVisitedAlternative::Plugin;
use strict;
use warnings;
use base qw! MT::Plugin !;
use MT::Entry;
use IO::File;

# Main routine
sub most_visited {
    my ( $ctx, $args, $cond ) = @_;

    my $plugin = MT->component( "MostVisitedAlternative" );
    my $config = $plugin->get_config_hash(
        "blog:" . $ctx->stash( "blog" )->id );

    # Location for config file of AWStats
    my $awstats_conf = $config->{awstats_conf};
    my $fh;
    eval {
        $fh = IO::File->new( $awstats_conf, "r" );
    };
    if ( !$fh or $@ ) {
        return $plugin->translate( "_MVA_CANNOT_OPEN_AWSTATS_CONF" );
    }

    # Location for AWStats log
    my $dir;
    {
        local $/ = undef;
        ( $dir ) = <$fh> =~ /^DirData\s*=\s*(.*)\s*$/m;
    }
    $dir =~ s/^['"]//;
    $dir =~ s/['"]$//;

    # Specify log file
    my $ym = sprintf "%02d%04d",
        &_get_ym( $ctx, "month" ), &_get_ym( $ctx, "year" );
    my ( $file ) = $awstats_conf =~ m!/awstats([^/]+)\.conf$!;

    # Open log file
    my $filename = "$dir/awstats$ym$file.txt";
    my $fh;
    eval {
        $fh = IO::File->new( $filename, "r" );
    };
    if ( !$fh or $@ ) {
        return $plugin->translate( "_MVA_CANNOT_OPEN_AWSTATS_LOG" );
    }

    # Ranking in log file
    my $txt;
    {
        local $/ = undef;
        ( $txt ) = <$fh> =~ /^BEGIN_SIDER \d+$(.*)^END_SIDER$/ms;
    }

    # Hash for getting entry_id from permalink
    my $link2id = &_get_link_id_table;

    # Array for ranking
    my @ranking;

    # Start handling
    for ( split /\n/, $txt ) {
        my ( $url, $count ) = split /\s+/;
        ( my ( $basename ) = $url =~ m!/([^/]+)\.(?:html|php)$! ) or next;
        ( my $data = $link2id->{ $basename } ) or next;

        $data->{count} = $count;
        push @ranking, $data;
    }

    @ranking = sort { $b->{count} <=> $a->{count} } @ranking;

    my $i = 0;
    my $imax = $args->{count} || 10;
    my $builder = $ctx->stash( "builder" );
    my $tokens = $ctx->stash( "tokens" );
    my $content = "";

    while ( ++$i <= $imax ) {
        my $data = $ranking[ $i - 1 ];

        $ctx->stash( most_visited_count => $data->{count} );
        $ctx->stash( most_visited_permalink => $data->{permalink} );
        $ctx->stash( most_visited_rank => $i );

        my $entry = MT::Entry->load( $data->{id} );
        local $ctx->{__stash}{entry} = $entry;
        local $ctx->{current_timestamp} = $entry->created_on;
        my $out = $builder->build( $ctx, $tokens );

        defined $out and $content .= $out;
    }

    $content;
}

# Year and month for log file
sub _get_ym {
    my ( $ctx, $kind ) = @_;

    # Return known value
    $ctx->stash( "most_visited_$kind" )
        and return $ctx->stash( "most_visited_$kind" );

    my $plugin = MT->component( "MostVisitedAlternative" );
    my $config = $plugin->get_config_hash(
        "blog:" . $ctx->stash( "blog" )->id );

    # Threshold to use log of last month
    my $awstats_threshold = $config->{awstats_threshold};
    $awstats_threshold =~ /\D/ and $awstats_threshold = 0;

    my @dt = localtime;
    my %ym;
    $ym{year} = $dt[5] + 1900;
    $ym{month} = $dt[4] + 1;

    # Read log of last month if today is before threshold
    if ( $awstats_threshold >= $dt[3] ) {
        if ( $ym{month} == 1 ) {
            $ym{year} -= 1;
            $ym{month} = 12;
        } else {
            $ym{month} -= 1;
        }
    }
    $ctx->stash( most_visited_year => $ym{year} );
    $ctx->stash( most_visited_month => $ym{month} );

    $ym{ $kind };
}

# Get entry_id from permalink
sub _get_link_id_table {
    my $entry_it = MT::Entry->load_iter( { blog_id => 4 } );
    my %link2id;
    while ( my $entry = $entry_it->next ) {
        my ( $basename ) = $entry->permalink =~ m!/([^/]+)\.(?:html|php)!;
        ( my $alt_basename = $basename ) =~ s/-/_/g;
        my $data = {
            id => $entry->id,
            permalink => $entry->permalink,
        };
        $link2id{ $basename } = $link2id{ $alt_basename } = $data;
    }
    \%link2id;
}

# Return each variable
sub most_visited_year { &_get_ym( shift, "year" ) }
sub most_visited_month { &_get_ym( shift, "month" ) }
sub most_visited_count { shift->stash( "most_visited_count" ) }
sub most_visited_link { shift->stash( "most_visited_permalink" ) }
sub most_visited_rank { shift->stash( "most_visited_rank" ) }

1;
